/*
 * File         : spmd_Procs.C
 * Author       : Guansong Zhang
 * Created      : Mon Oct 13 12:06:49 1997
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/04/07 19:40:32 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "spmd_Procs.h"
#include "adlib.h"
#include "iostream.h"

extern jfieldID hCPPObjID;
#include "jctable.h"
extern jctable<Dimension> spmd_Dimension_Table;
extern jctable<Group> spmd_Group_Table;

Dimension * spmd_Dimension_ResolvePtr(JNIEnv *, jobject);

/*ResolvePtr
 */
Group * spmd_Group_ResolvePtr(JNIEnv *, jobject);

/*
 * Class:     spmd_Procs
 * Method:    constructor
 * Signature: ([I[I)V
 */
JNIEXPORT void JNICALL Java_spmd_Procs_constructor
  (JNIEnv *env, jobject jthis, jintArray grid, jintArray idMap)
{

  int rank;
  int * dimSize;
  jint * jdimSize;

  int pnum;
  int * ids;
  jint * jids;

  //java null is zero in C++;

  if (grid == 0) {
    rank=0;
    dimSize=0;
  } else {
    rank=(env->GetArrayLength(grid));
    dimSize=new int [rank]; // it is deleted outside this block;

    jdimSize=env->GetIntArrayElements(grid, 0);
    for (int i=0; i<rank; i++) 
      *(dimSize+i)=*(jdimSize+i);
    env->ReleaseIntArrayElements(grid, jdimSize, 0);
  }

  if (idMap == 0) {
    ids=0;
  } else { 
    pnum=(env->GetArrayLength(idMap));
    ids=new int[pnum]; // it is deleted outside this block;

    jids=env->GetIntArrayElements(idMap, 0);
    for (int i=0; i<pnum; i++) 
      *(ids+i)=*(jids+i);
    env->ReleaseIntArrayElements(idMap, jids, 0);
  }

  Procs *procsPtr = new Procs(rank, dimSize, ids);

  if (procsPtr==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  Group *cppGroup=new Group(*procsPtr);

  int pos = env->GetIntField(jthis, hCPPObjID);
  delete (spmd_Group_Table.ObjPtr[pos]);

  spmd_Group_Table.ObjPtr[pos]=cppGroup;

  if (spmd_Group_Table.ObjPtr[pos]==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  //spmd_Group_IncRefCntr(pos);

  delete [] dimSize; 
  delete [] ids;

  env->SetIntField(jthis, hCPPObjID, pos);

}

/*
 * Class:     spmd_Procs
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Procs_finalize
  (JNIEnv * env, jobject jthis)
{
  int pos = env->GetIntField(jthis, hCPPObjID);
  int rc=spmd_Group_Table.DecRefCntr(pos);
  if (rc==0) {
    Group * groupPtr=spmd_Group_Table.ObjPtr[pos];
    Procs * procsPtr=(Procs *)groupPtr->prc();
    delete procsPtr;
    delete groupPtr;
    groupPtr=NULL;
  }
};

/*
 * Class:     spmd_Procs
 * Method:    dimension
 * Signature: (I)Lspmd/Dimension;
 */
JNIEXPORT jobject JNICALL Java_spmd_Procs_dimension
  (JNIEnv * env, jobject jthis, jint d)
{

  //Dimension *newCPPdimPtr=new Dimension();

  Dimension CPPdim = (spmd_Group_ResolvePtr(env, jthis)->prc())->dim(d);
  
  //long pos=spmd_Dimension_NewHandle(newCPPdimPtr);

  //printf("dim pos %ld, address %x\n", pos, spmd_Dimension_ObjPtr[pos]);
  //printf("is this dim? crd %d\n", spmd_Dimension_ObjPtr[pos]->crd());

  jclass ccDimension=NULL;
  ccDimension=env->FindClass("spmd/Dimension");
  if (env->ExceptionOccurred()) return 0;
  jmethodID handleConstructorID = env->GetMethodID(ccDimension, "<init>", "()V");
  jobject jDimension=env->NewObject(ccDimension, handleConstructorID);

  Dimension * newCPPdimPtr=spmd_Dimension_ResolvePtr(env, jDimension);

  *newCPPdimPtr=CPPdim;

  return(jDimension);
};

